package de.ozgcloud.eingang.formcycle;

import static org.assertj.core.api.Assertions.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.eingang.common.formdata.ServiceKonto.PostfachAddress;
import de.ozgcloud.eingang.common.formdata.ServiceKontoTestFactory;
import de.ozgcloud.eingang.common.formdata.StringBasedIdentifier;
import de.ozgcloud.eingang.semantik.common.ServiceKontoFactory;

class FormCycleServiceKontoMapperTest {

	private final FormCycleServiceKontoMapper mapper = Mappers.getMapper(FormCycleServiceKontoMapper.class);

	@DisplayName("From grpc serviceKonto")
	@Nested
	class TestFromGrpcServiceKonto {

		@Test
		void shouldMapType() {
			var result = mapper.fromGrpc(FormCycleServiceKontoTestFactory.create());

			assertThat(result.getType()).isEqualTo(ServiceKontoFactory.POSTFACH_TYPE_BAYERN_ID);
		}

		@DisplayName("trustLevel")
		@Nested
		class TestTrustLevel {

			@Test
			void shouldMap() {
				var serviceKonto = mapper.fromGrpc(FormCycleServiceKontoTestFactory.create());

				assertThat(serviceKonto.getTrustLevel()).isEqualTo(ServiceKontoTestFactory.TRUST_LEVEL);
			}

			@Test
			void shouldMapAsNullIfEmpty() {
				var serviceKontoWithEmptyTrustLevel = FormCycleServiceKontoTestFactory.createBuilder().setTrustLevel(StringUtils.EMPTY).build();

				var serviceKonto = mapper.fromGrpc(serviceKontoWithEmptyTrustLevel);

				assertThat(serviceKonto.getTrustLevel()).isNull();
			}
		}

		@Test
		void shouldNotPostfachAddressIfNotExists() {
			var serviceKonto = mapper.fromGrpc(FormCycleServiceKontoTestFactory.createBuilder().clearAddress().build());

			assertThat(serviceKonto.getPostfachAddresses()).isEmpty();
		}

		@DisplayName("postfachAddress")
		@Nested
		class TestPostfachAddress {

			@Test
			void shouldMapPostkorbId() {
				var postfachAddress = mapPostfachAddressFromGrpc();

				assertThat(((StringBasedIdentifier) postfachAddress.getIdentifier()).getPostfachId())
						.isEqualTo(FormCyclePostfachAddressTestFactory.POSTKORB_ID);
			}

			@Test
			void shouldMapIdentifier() {
				var postfachAddress = mapPostfachAddressFromGrpc();

				assertThat(getIdentifier(postfachAddress)).isEqualTo(FormCyclePostfachAddressTestFactory.POSTKORB_ID);
			}

			private String getIdentifier(PostfachAddress postfachAddress) {
				return ((StringBasedIdentifier) postfachAddress.getIdentifier()).getPostfachId();
			}

			@Test
			void shouldMapVersion() {
				var postfachAddress = mapPostfachAddressFromGrpc();

				assertThat(postfachAddress.getVersion()).isEqualTo(FormCyclePostfachAddressTestFactory.VERSION);
			}

			private PostfachAddress mapPostfachAddressFromGrpc() {
				return mapper.fromGrpc(FormCycleServiceKontoTestFactory.create()).getPostfachAddresses().get(0);
			}
		}
	}
}